@@ -68,7 +68,7 @@ web_radiobutton_toggled_cb (GtkWidget *togglebutton, GnomeDACapplet *capplet)
 {
     gint index;
     GnomeDAWebItem *item;
-    gchar *command;
+    const gchar *command;
     GError *error = NULL;
 
     index = gtk_combo_box_get_active (GTK_COMBO_BOX (capplet->web_combo_box));
@@ -77,10 +77,10 @@ web_radiobutton_toggled_cb (GtkWidget *togglebutton, GnomeDACapplet *capplet)
 	return;
 
     item = (GnomeDAWebItem *) g_list_nth_data (capplet->web_browsers, index);
-    if (item == NULL) {
-	command = "";
-    }
-    else if (togglebutton == capplet->new_win_radiobutton) {
+    if (item == NULL)
+	return;
+
+    if (togglebutton == capplet->new_win_radiobutton) {
 	command = item->win_command;
     }
     else if (togglebutton == capplet->new_tab_radiobutton) {
@@ -259,11 +259,11 @@ static struct {
 static void
 theme_changed_cb (GtkIconTheme *theme, GnomeDACapplet *capplet)
 {
-    GtkWidget *icon;
+    GObject *icon;
     gint i;
 
     for (i = 0; i < G_N_ELEMENTS (icons); i++) {
-	icon = glade_xml_get_widget (capplet->xml, icons[i].name);
+	icon = gtk_builder_get_object (capplet->builder, icons[i].name);
 	set_icon (GTK_IMAGE (icon), theme, icons[i].icon);
     }
 
@@ -492,7 +492,6 @@ web_combo_conv_from_widget (GConfPropertyEditor *peditor, const GConfValue *valu
     if (!item)
     {
         /* if item was not found, this is probably the "Custom" item */
-
         /* XXX: returning "" as the value here is not ideal, but required to
          * prevent the combo box from jumping back to the previous value if the
          * user has selected Custom */
@@ -575,6 +574,50 @@ combo_conv_from_widget (GConfPropertyEditor *peditor, const GConfValue *value)
     }
 }
 
+static GConfValue*
+combo_conv_from_widget_term_flag (GConfPropertyEditor *peditor, const GConfValue *value)
+{
+    GConfValue *ret;
+    GList *handlers;
+    gint index;
+    GnomeDATermItem *item;
+
+    g_object_get (G_OBJECT (peditor), "data", &handlers, NULL);
+    index = gconf_value_get_int (value);
+
+    item = g_list_nth_data (handlers, index);
+    ret = gconf_value_new (GCONF_VALUE_STRING);
+
+    if (!item)
+    {
+        /* if item was not found, this is probably the "Custom" item */
+
+        /* XXX: returning "" as the value here is not ideal, but required to
+         * prevent the combo box from jumping back to the previous value if the
+         * user has selected Custom */
+        gconf_value_set_string (ret, "");
+        return ret;
+    }
+    else
+    {
+        gconf_value_set_string (ret, item->exec_flag);
+        return ret;
+    }
+}
+
+static GConfValue*
+combo_conv_to_widget_term_flag (GConfPropertyEditor *peditor, const GConfValue *value)
+{
+    GConfValue *ret;
+    GtkComboBox *combo;
+
+    combo = GTK_COMBO_BOX (gconf_property_editor_get_ui_control (peditor));
+
+    ret = gconf_value_new (GCONF_VALUE_INT);
+    gconf_value_set_int (ret, gtk_combo_box_get_active (combo));
+    return ret;
+}
+
 static gboolean
 is_separator (GtkTreeModel *model, GtkTreeIter *iter, gpointer sep_index)
 {
@@ -649,19 +692,30 @@ fill_combo_box (GtkIconTheme *theme, GtkComboBox *combo_box, GList *app_list)
 			-1);
 }
 
+static GtkWidget*
+_gtk_builder_get_widget (GtkBuilder *builder, const gchar *name)
+{
+    return GTK_WIDGET (gtk_builder_get_object (builder, name));
+}
+
+
 static void
 show_dialog (GnomeDACapplet *capplet, const gchar *start_page)
 {
     GObject *obj;
+    GtkBuilder *builder;
+    guint builder_result;
 
-    if (g_file_test (GNOMECC_GLADE_DIR "/gnome-default-applications-properties.glade", G_FILE_TEST_EXISTS) != FALSE) {
-	capplet->xml = glade_xml_new (GNOMECC_GLADE_DIR "/gnome-default-applications-properties.glade", NULL, NULL);
+    capplet->builder = builder = gtk_builder_new ();
+
+    if (g_file_test (GNOMECC_UI_DIR "/gnome-default-applications-properties.ui", G_FILE_TEST_EXISTS) != FALSE) {
+	builder_result = gtk_builder_add_from_file (builder, GNOMECC_UI_DIR "/gnome-default-applications-properties.ui", NULL);
     }
     else {
-	capplet->xml = glade_xml_new ("./gnome-default-applications-properties.glade", NULL, NULL);
+	builder_result = gtk_builder_add_from_file (builder, "./gnome-default-applications-properties.ui", NULL);
     }
 
-    if (capplet->xml == NULL) {
+    if (builder_result == 0) {
 	GtkWidget *dialog;
 
 	dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
@@ -675,43 +729,43 @@ show_dialog (GnomeDACapplet *capplet, const gchar *start_page)
 	exit (EXIT_FAILURE);
     }
 
-    capplet->window = glade_xml_get_widget (capplet->xml, "preferred_apps_dialog");
+    capplet->window = _gtk_builder_get_widget (builder,"preferred_apps_dialog");
     g_signal_connect (capplet->window, "response", G_CALLBACK (close_cb), NULL);
 
-    capplet->web_browser_command_entry = glade_xml_get_widget (capplet->xml, "web_browser_command_entry");
-    capplet->web_browser_command_label = glade_xml_get_widget (capplet->xml, "web_browser_command_label");
-    capplet->web_browser_terminal_checkbutton = glade_xml_get_widget (capplet->xml, "web_browser_terminal_checkbutton");
-    capplet->default_radiobutton = glade_xml_get_widget (capplet->xml, "web_browser_default_radiobutton");
-    capplet->new_win_radiobutton = glade_xml_get_widget (capplet->xml, "web_browser_new_win_radiobutton");
-    capplet->new_tab_radiobutton = glade_xml_get_widget (capplet->xml, "web_browser_new_tab_radiobutton");
+    capplet->web_browser_command_entry = _gtk_builder_get_widget (builder, "web_browser_command_entry");
+    capplet->web_browser_command_label = _gtk_builder_get_widget (builder, "web_browser_command_label");
+    capplet->web_browser_terminal_checkbutton = _gtk_builder_get_widget(builder, "web_browser_terminal_checkbutton");
+    capplet->default_radiobutton = _gtk_builder_get_widget (builder, "web_browser_default_radiobutton");
+    capplet->new_win_radiobutton = _gtk_builder_get_widget (builder, "web_browser_new_win_radiobutton");
+    capplet->new_tab_radiobutton = _gtk_builder_get_widget (builder, "web_browser_new_tab_radiobutton");
 
-    capplet->mail_reader_command_entry = glade_xml_get_widget (capplet->xml, "mail_reader_command_entry");
-    capplet->mail_reader_command_label = glade_xml_get_widget (capplet->xml, "mail_reader_command_label");
-    capplet->mail_reader_terminal_checkbutton = glade_xml_get_widget (capplet->xml, "mail_reader_terminal_checkbutton");
+    capplet->mail_reader_command_entry = _gtk_builder_get_widget (builder, "mail_reader_command_entry");
+    capplet->mail_reader_command_label = _gtk_builder_get_widget (builder, "mail_reader_command_label");
+    capplet->mail_reader_terminal_checkbutton = _gtk_builder_get_widget (builder, "mail_reader_terminal_checkbutton");
 
-    capplet->terminal_command_entry = glade_xml_get_widget (capplet->xml, "terminal_command_entry");
-    capplet->terminal_command_label = glade_xml_get_widget (capplet->xml, "terminal_command_label");
-    capplet->terminal_exec_flag_entry = glade_xml_get_widget (capplet->xml, "terminal_exec_flag_entry");
-    capplet->terminal_exec_flag_label = glade_xml_get_widget (capplet->xml, "terminal_exec_flag_label");
+    capplet->terminal_command_entry = _gtk_builder_get_widget (builder, "terminal_command_entry");
+    capplet->terminal_command_label = _gtk_builder_get_widget (builder, "terminal_command_label");
+    capplet->terminal_exec_flag_entry = _gtk_builder_get_widget (builder, "terminal_exec_flag_entry");
+    capplet->terminal_exec_flag_label = _gtk_builder_get_widget (builder, "terminal_exec_flag_label");
 
-    capplet->media_player_command_entry = glade_xml_get_widget (capplet->xml, "media_player_command_entry");
-    capplet->media_player_command_label = glade_xml_get_widget (capplet->xml, "media_player_command_label");
-    capplet->media_player_terminal_checkbutton = glade_xml_get_widget (capplet->xml, "media_player_terminal_checkbutton");
+    capplet->media_player_command_entry = _gtk_builder_get_widget (builder, "media_player_command_entry");
+    capplet->media_player_command_label = _gtk_builder_get_widget (builder, "media_player_command_label");
+    capplet->media_player_terminal_checkbutton = _gtk_builder_get_widget (builder, "media_player_terminal_checkbutton");
 
-    capplet->visual_command_entry = glade_xml_get_widget (capplet->xml, "visual_command_entry");
-    capplet->visual_command_label = glade_xml_get_widget (capplet->xml, "visual_command_label");
-    capplet->visual_startup_checkbutton = glade_xml_get_widget (capplet->xml, "visual_start_checkbutton");
+    capplet->visual_command_entry = _gtk_builder_get_widget (builder, "visual_command_entry");
+    capplet->visual_command_label = _gtk_builder_get_widget (builder, "visual_command_label");
+    capplet->visual_startup_checkbutton = _gtk_builder_get_widget (builder, "visual_start_checkbutton");
 
-    capplet->mobility_command_entry = glade_xml_get_widget (capplet->xml, "mobility_command_entry");
-    capplet->mobility_command_label = glade_xml_get_widget (capplet->xml, "mobility_command_label");
-    capplet->mobility_startup_checkbutton = glade_xml_get_widget (capplet->xml, "mobility_start_checkbutton");
+    capplet->mobility_command_entry = _gtk_builder_get_widget (builder, "mobility_command_entry");
+    capplet->mobility_command_label = _gtk_builder_get_widget (builder, "mobility_command_label");
+    capplet->mobility_startup_checkbutton = _gtk_builder_get_widget (builder, "mobility_start_checkbutton");
 
-    capplet->web_combo_box = glade_xml_get_widget (capplet->xml, "web_browser_combobox");
-    capplet->mail_combo_box = glade_xml_get_widget (capplet->xml, "mail_reader_combobox");
-    capplet->term_combo_box = glade_xml_get_widget (capplet->xml, "terminal_combobox");
-    capplet->media_combo_box = glade_xml_get_widget (capplet->xml, "media_player_combobox");
-    capplet->visual_combo_box = glade_xml_get_widget (capplet->xml, "visual_combobox");
-    capplet->mobility_combo_box = glade_xml_get_widget (capplet->xml, "mobility_combobox");
+    capplet->web_combo_box = _gtk_builder_get_widget (builder, "web_browser_combobox");
+    capplet->mail_combo_box = _gtk_builder_get_widget (builder, "mail_reader_combobox");
+    capplet->term_combo_box = _gtk_builder_get_widget (builder, "terminal_combobox");
+    capplet->media_combo_box = _gtk_builder_get_widget (builder, "media_player_combobox");
+    capplet->visual_combo_box = _gtk_builder_get_widget (builder, "visual_combobox");
+    capplet->mobility_combo_box = _gtk_builder_get_widget (builder, "mobility_combobox");
 
     g_signal_connect (capplet->window, "screen-changed", G_CALLBACK (screen_changed_cb), capplet);
     screen_changed_cb (capplet->window, gdk_screen_get_default (), capplet);
@@ -805,6 +859,14 @@ show_dialog (GnomeDACapplet *capplet, const gchar *start_page)
         "data", capplet->terminals,
         NULL);
 
+    gconf_peditor_new_combo_box (NULL,
+        DEFAULT_APPS_KEY_TERMINAL_EXEC_ARG,
+        capplet->term_combo_box,
+        "conv-from-widget-cb", combo_conv_from_widget_term_flag,
+        "conv-to-widget-cb", combo_conv_to_widget_term_flag,
+        "data", capplet->terminals,
+        NULL);
+
     gconf_peditor_new_string (NULL,
         DEFAULT_APPS_KEY_TERMINAL_EXEC,
         capplet->terminal_command_entry,
@@ -863,12 +925,13 @@ show_dialog (GnomeDACapplet *capplet, const gchar *start_page)
 
         page_name = g_strconcat (start_page, "_vbox", NULL);
 
-        w = glade_xml_get_widget (capplet->xml, page_name);
+        w = _gtk_builder_get_widget (builder, page_name);
         if (w != NULL) {
             GtkNotebook *nb;
             gint pindex;
 
-            nb = GTK_NOTEBOOK (glade_xml_get_widget (capplet->xml, "preferred_apps_notebook"));
+            nb = GTK_NOTEBOOK (_gtk_builder_get_widget (builder,
+                                                        "preferred_apps_notebook"));
             pindex = gtk_notebook_page_num (nb, w);
             if (pindex != -1)
                 gtk_notebook_set_current_page (nb, pindex);
@@ -903,10 +966,14 @@ main (int argc, char **argv)
 
     capplet_init (context, &argc, &argv);
 
-    glade_init ();
-
     capplet = g_new0 (GnomeDACapplet, 1);
     capplet->gconf = gconf_client_get_default ();
+    gconf_client_add_dir (capplet->gconf, "/desktop/gnome/url-handlers",
+                          GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
+    gconf_client_add_dir (capplet->gconf, 
+                          "/desktop/gnome/applications",
+                          GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
+
     gnome_da_xml_load_list (capplet);
 
     show_dialog (capplet, start_page);
