@@ -522,33 +522,26 @@ find_section (GtkTreeModel *model,
               GtkTreeIter  *iter,
 	      const char   *title)
 {
-  gint i, j;
-  gboolean found;
+  gboolean success;
 
-  i = gtk_tree_model_iter_n_children (model, NULL);
-  found = FALSE;
-  gtk_tree_model_get_iter_first (model, iter);
-  for (j = 0; j < i; j++)
+  success = gtk_tree_model_get_iter_first (model, iter);
+  while (success)
     {
       char *description = NULL;
 
-      gtk_tree_model_iter_next (model, iter);
       gtk_tree_model_get (model, iter,
 			  DESCRIPTION_COLUMN, &description,
 			  -1);
+
       if (g_strcmp0 (description, title) == 0)
-        {
-	  found = TRUE;
-     	  break;
-        }
-    }
-  if (!found)
-    {
-      gtk_tree_store_append (GTK_TREE_STORE (model), iter, NULL);
-      gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-			  DESCRIPTION_COLUMN, title,
-			  -1);
+        return;
+      success = gtk_tree_model_iter_next (model, iter);
     }
+
+    gtk_tree_store_append (GTK_TREE_STORE (model), iter, NULL);
+    gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+                        DESCRIPTION_COLUMN, title,
+                        -1);
 }
 
 static void
@@ -560,7 +553,6 @@ append_keys_to_tree (GtkBuilder         *builder,
   GtkTreeIter parent_iter, iter;
   GtkTreeModel *model;
   gint i, j;
-  gint rows_before;
 
   client = gconf_client_get_default ();
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (gtk_builder_get_object (builder, "shortcut_treeview")));
@@ -576,7 +568,6 @@ append_keys_to_tree (GtkBuilder         *builder,
    * then we need to scroll now */
   ensure_scrollbar (builder, i - 1);
 
-  rows_before = i;
   for (j = 0; keys_list[j].name != NULL; j++)
     {
       GConfEntry *entry;
@@ -694,7 +685,7 @@ append_keys_to_tree (GtkBuilder         *builder,
   g_object_unref (client);
 
   /* Don't show an empty section */
-  if (i == rows_before)
+  if (gtk_tree_model_iter_n_children (model, &parent_iter) == 0)
     gtk_tree_store_remove (GTK_TREE_STORE (model), &parent_iter);
 
   if (i == 0)
@@ -821,10 +812,22 @@ parse_start_tag (GMarkupParseContext *ctx,
   g_array_append_val (keylist->entries, key);
 }
 
+static gboolean
+strv_contains (char **strv,
+	       char  *str)
+{
+  char **p = strv;
+  for (p = strv; *p; p++)
+    if (strcmp (*p, str) == 0)
+      return TRUE;
+
+  return FALSE;
+}
+
 static void
 append_keys_to_tree_from_file (GtkBuilder *builder,
 			       const char *filename,
-			       const char *wm_name)
+			       char      **wm_keybindings)
 {
   GMarkupParseContext *ctx;
   GMarkupParser parser = { parse_start_tag, NULL, NULL, NULL, NULL };
@@ -865,7 +868,7 @@ append_keys_to_tree_from_file (GtkBuilder *builder,
   /* If there's no keys to add, or the settings apply to a window manager
    * that's not the one we're running */
   if (keylist->entries->len == 0
-      || (keylist->wm_name != NULL && !g_str_equal (wm_name, keylist->wm_name))
+      || (keylist->wm_name != NULL && !strv_contains (wm_keybindings, keylist->wm_name))
       || keylist->name == NULL)
     {
       g_free (keylist->name);
@@ -956,12 +959,15 @@ append_keys_to_tree_from_gconf (GtkBuilder *builder, const gchar *gconf_path)
 }
 
 static void
-reload_key_entries (gpointer wm_name, GtkBuilder *builder)
+reload_key_entries (GtkBuilder *builder)
 {
+  gchar **wm_keybindings;
   GDir *dir;
   const char *name;
   GList *list, *l;
 
+  wm_keybindings = wm_common_get_current_keybindings();
+
   clear_old_model (builder);
 
   dir = g_dir_open (GNOMECC_KEYBINDINGS_DIR, 0, NULL);
@@ -984,7 +990,7 @@ reload_key_entries (gpointer wm_name, GtkBuilder *builder)
         gchar *path;
 
 	path = g_build_filename (GNOMECC_KEYBINDINGS_DIR, l->data, NULL);
-        append_keys_to_tree_from_file (builder, path, wm_name);
+        append_keys_to_tree_from_file (builder, path, wm_keybindings);
 	g_free (l->data);
 	g_free (path);
     }
@@ -996,6 +1002,8 @@ reload_key_entries (gpointer wm_name, GtkBuilder *builder)
    * such keys not show up in the custom section.
    */
   append_keys_to_tree_from_gconf (builder, GCONF_BINDING_DIR);
+
+  g_strfreev (wm_keybindings);
 }
 
 static void
@@ -1004,9 +1012,7 @@ key_entry_controlling_key_changed (GConfClient *client,
 				   GConfEntry  *entry,
 				   gpointer     user_data)
 {
-  gchar *wm_name = wm_common_get_current_window_manager();
-  reload_key_entries (wm_name, user_data);
-  g_free (wm_name);
+  reload_key_entries (user_data);
 }
 
 static gboolean
@@ -1452,6 +1458,10 @@ remove_custom_shortcut (GtkTreeModel *model, GtkTreeIter *iter)
   base = g_path_get_dirname (key->gconf_key);
   gconf_client_recursive_unset (client, base, 0, NULL);
   g_free (base);
+  /* suggest sync now so the unset directory actually gets dropped;
+   * if we don't do this we may end up with 'zombie' shortcuts when
+   * restarting the app */
+  gconf_client_suggest_sync (client, NULL);
   g_object_unref (client);
 
   g_free (key->gconf_key);
@@ -1800,7 +1810,6 @@ setup_dialog (GtkBuilder *builder)
   GtkTreeViewColumn *column;
   GtkWidget *widget;
   GtkTreeView *treeview;
-  gchar *wm_name;
   GtkTreeSelection *selection;
   GSList *allowed_keys;
 
@@ -1857,9 +1866,7 @@ setup_dialog (GtkBuilder *builder)
 			   builder, NULL, NULL);
 
   /* set up the dialog */
-  wm_name = wm_common_get_current_window_manager();
-  reload_key_entries (wm_name, builder);
-  g_free (wm_name);
+  reload_key_entries (builder);
 
   widget = _gtk_builder_get_widget (builder, "gnome-keybinding-dialog");
   capplet_set_icon (widget, "preferences-desktop-keyboard-shortcuts");
@@ -1900,6 +1907,12 @@ setup_dialog (GtkBuilder *builder)
                                 GTK_WINDOW (widget));
 }
 
+static void
+on_window_manager_change (const char *wm_name, GtkBuilder *builder)
+{
+  reload_key_entries (builder);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1921,7 +1934,7 @@ main (int argc, char *argv[])
   if (!builder) /* Warning was already printed to console */
     exit (EXIT_FAILURE);
 
-  wm_common_register_window_manager_change ((GFunc) reload_key_entries, builder);
+  wm_common_register_window_manager_change ((GFunc) on_window_manager_change, builder);
   setup_dialog (builder);
 
   gtk_main ();
