@@ -93,6 +93,9 @@ check_font_contain_text (FT_Face face, const gchar *text)
     return TRUE;
 }
 
+
+static gboolean expose_event(GtkWidget *widget, GdkEventExpose *event, GdkPixmap *pixmap);
+
 static GdkPixmap *
 create_text_pixmap(GtkWidget *drawing_area, FT_Face face)
 {
@@ -109,6 +112,7 @@ create_text_pixmap(GtkWidget *drawing_area, FT_Face face)
     XftFont *font;
     gint *sizes = NULL, n_sizes, alpha_size;
     FcCharSet *charset = NULL;
+    GdkWindow *window = gtk_widget_get_window (drawing_area);
 
     text = pango_language_get_sample_string(NULL);
     if (! check_font_contain_text (face, text))
@@ -118,13 +122,10 @@ create_text_pixmap(GtkWidget *drawing_area, FT_Face face)
 
     textlen = strlen(text);
 
-    /* create pixmap */
-    gtk_widget_realize(drawing_area);
-
     /* create the XftDraw */
-    xdisplay = GDK_PIXMAP_XDISPLAY(drawing_area->window);
-    xvisual = GDK_VISUAL_XVISUAL(gdk_drawable_get_visual(drawing_area->window));
-    xcolormap = GDK_COLORMAP_XCOLORMAP(gdk_drawable_get_colormap(drawing_area->window));
+    xdisplay = GDK_PIXMAP_XDISPLAY(window);
+    xvisual = GDK_VISUAL_XVISUAL(gdk_drawable_get_visual(window));
+    xcolormap = GDK_COLORMAP_XCOLORMAP(gdk_drawable_get_colormap(window));
     XftColorAllocName(xdisplay, xvisual, xcolormap, "black", &colour);
 
     /* work out what sizes to render */
@@ -187,11 +188,11 @@ create_text_pixmap(GtkWidget *drawing_area, FT_Face face)
 
     /* create pixmap */
     gtk_widget_set_size_request(drawing_area, pixmap_width, pixmap_height);
-    pixmap = gdk_pixmap_new(drawing_area->window,
+    pixmap = gdk_pixmap_new(window,
 			    pixmap_width, pixmap_height, -1);
     if (!pixmap)
 	goto end;
-    gdk_draw_rectangle(pixmap, drawing_area->style->white_gc,
+    gdk_draw_rectangle(pixmap, gtk_widget_get_style(drawing_area)->white_gc,
 		       TRUE, 0, 0, pixmap_width, pixmap_height);
 
     xdrawable = GDK_DRAWABLE_XID(pixmap);
@@ -213,6 +214,9 @@ create_text_pixmap(GtkWidget *drawing_area, FT_Face face)
 	XftFontClose(xdisplay, font);
     }
 
+    g_signal_connect(drawing_area, "expose-event", G_CALLBACK(expose_event),
+                     pixmap);
+
  end:
     g_free(sizes);
     FcCharSetDestroy (charset);
@@ -372,8 +376,8 @@ add_face_info(GtkWidget *table, gint *row_p, const gchar *uri, FT_Face face)
 static gboolean
 expose_event(GtkWidget *widget, GdkEventExpose *event, GdkPixmap *pixmap)
 {
-    gdk_draw_drawable(widget->window,
-		      widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+    gdk_draw_drawable(gtk_widget_get_window (widget),
+		      gtk_widget_get_style (widget)->fg_gc[gtk_widget_get_state (widget)],
 		      pixmap,
 		      event->area.x, event->area.y,
 		      event->area.x, event->area.y,
@@ -503,7 +507,6 @@ main(int argc, char **argv)
     GtkWidget *window, *hbox, *table, *swin, *drawing_area;
     GdkPixmap *pixmap;
     GdkColor white = { 0, 0xffff, 0xffff, 0xffff };
-    gint height;
     GtkWidget *button, *align;
 
     bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -564,16 +567,12 @@ main(int argc, char **argv)
     gtk_widget_modify_bg(drawing_area, GTK_STATE_NORMAL, &white);
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin),
 					  drawing_area);
-
-    pixmap = create_text_pixmap(drawing_area, face);
+    g_signal_connect (drawing_area, "realize", create_text_pixmap, face);
 
     /* set the minimum size on the scrolled window to prevent
      * unnecessary scrolling */
-    gdk_drawable_get_size (GDK_DRAWABLE (pixmap), NULL, &height);
-    gtk_widget_set_size_request(swin, 500, height + 30);
+    gtk_widget_set_size_request(swin, 500, -1);
 
-    g_signal_connect(drawing_area, "expose_event",
-		     G_CALLBACK(expose_event), pixmap);
     g_signal_connect(window, "destroy", G_CALLBACK(gtk_main_quit), NULL);
 
     table = gtk_table_new(1, 2, FALSE);
